@@ -20,20 +20,52 @@
  */
 package bitronix.tm;
 
-import bitronix.tm.internal.*;
+import bitronix.tm.internal.BitronixMultiSystemException;
+import bitronix.tm.internal.BitronixRollbackException;
+import bitronix.tm.internal.BitronixRollbackSystemException;
+import bitronix.tm.internal.BitronixSystemException;
+import bitronix.tm.internal.BitronixXAException;
+import bitronix.tm.internal.TransactionStatusChangeListener;
+import bitronix.tm.internal.XAResourceHolderState;
+import bitronix.tm.internal.XAResourceManager;
 import bitronix.tm.journal.Journal;
-import bitronix.tm.twopc.*;
 import bitronix.tm.resource.ResourceRegistrar;
 import bitronix.tm.resource.common.XAResourceHolder;
-import bitronix.tm.utils.*;
-import org.slf4j.LoggerFactory;
+import bitronix.tm.timer.TaskScheduler;
+import bitronix.tm.twopc.Committer;
+import bitronix.tm.twopc.PhaseException;
+import bitronix.tm.twopc.Preparer;
+import bitronix.tm.twopc.Rollbacker;
+import bitronix.tm.twopc.executor.Executor;
+import bitronix.tm.utils.Decoder;
+import bitronix.tm.utils.ExceptionUtils;
+import bitronix.tm.utils.ManagementRegistrar;
+import bitronix.tm.utils.MonotonicClock;
+import bitronix.tm.utils.Scheduler;
+import bitronix.tm.utils.StackTrace;
+import bitronix.tm.utils.Uid;
+import bitronix.tm.utils.UidGenerator;
 import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-import javax.transaction.*;
+import javax.transaction.HeuristicCommitException;
+import javax.transaction.HeuristicMixedException;
+import javax.transaction.HeuristicRollbackException;
+import javax.transaction.RollbackException;
+import javax.transaction.Status;
+import javax.transaction.Synchronization;
+import javax.transaction.SystemException;
+import javax.transaction.Transaction;
 import javax.transaction.xa.XAException;
 import javax.transaction.xa.XAResource;
 import java.io.IOException;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Date;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
 
 /**
  * Implementation of {@link Transaction}.
@@ -44,20 +76,25 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
 
     private final static Logger log = LoggerFactory.getLogger(BitronixTransaction.class);
 
+    private final XAResourceManager resourceManager;
+    private final Scheduler<Synchronization> synchronizationScheduler = new Scheduler<Synchronization>();
+    private final List<TransactionStatusChangeListener> transactionStatusListeners = new ArrayList<TransactionStatusChangeListener>();
     private volatile int status = Status.STATUS_NO_TRANSACTION;
-    private XAResourceManager resourceManager;
-    private Scheduler synchronizationScheduler = new Scheduler();
-    private List transactionStatusListeners = new ArrayList();
-    private boolean timeout = false;
-    private Date timeoutDate;
+    private volatile boolean timeout = false;
+    private volatile Date timeoutDate;
 
-    private Preparer preparer = new Preparer(TransactionManagerServices.getExecutor());
-    private Committer committer = new Committer(TransactionManagerServices.getExecutor());
-    private Rollbacker rollbacker = new Rollbacker(TransactionManagerServices.getExecutor());
+    private final Executor executor = TransactionManagerServices.getExecutor();
+    private final TaskScheduler taskScheduler = TransactionManagerServices.getTaskScheduler();
+    private final Journal journal = TransactionManagerServices.getJournal();
+
+    private final Preparer preparer = new Preparer(executor);
+    private final Committer committer = new Committer(executor);
+    private final Rollbacker rollbacker = new Rollbacker(executor);
 
     /* management */
-    private String threadName;
-    private Date startDate;
+    private volatile String threadName;
+    private volatile Date startDate;
+    private volatile StackTrace activationStackTrace;
 
 
     public BitronixTransaction() {
@@ -92,12 +129,15 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
         try {
             resourceManager.enlist(resourceHolderState);
         } catch (XAException ex) {
+            String extraErrorDetails = TransactionManagerServices.getExceptionAnalyzer().extractExtraXAExceptionDetails(ex);
             if (BitronixXAException.isUnilateralRollback(ex)) {
                 // if the resource unilaterally rolled back, the transaction will never be able to commit -> mark it as rollback only
                 setStatus(Status.STATUS_MARKED_ROLLBACK);
-                throw new BitronixRollbackException("resource " + resourceHolderState + " unilaterally rolled back, error=" + Decoder.decodeXAExceptionErrorCode(ex), ex);
+                throw new BitronixRollbackException("resource " + resourceHolderState + " unilaterally rolled back, error=" +
+                        Decoder.decodeXAExceptionErrorCode(ex) + (extraErrorDetails == null ? "" : ", extra error=" + extraErrorDetails), ex);
             }
-            throw new BitronixSystemException("cannot enlist " + resourceHolderState + ", error=" + Decoder.decodeXAExceptionErrorCode(ex), ex);
+            throw new BitronixSystemException("cannot enlist " + resourceHolderState + ", error=" +
+                    Decoder.decodeXAExceptionErrorCode(ex) + (extraErrorDetails == null ? "" : ", extra error=" + extraErrorDetails), ex);
         }
 
         resourceHolder.putXAResourceHolderState(resourceHolderState.getXid(), resourceHolderState);
@@ -116,14 +156,12 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
         if (resourceHolder == null)
             throw new BitronixSystemException("unknown XAResource " + xaResource + ", it does not belong to a registered resource");
 
-        Map statesForGtrid = resourceHolder.getXAResourceHolderStatesForGtrid(resourceManager.getGtrid());
-        Iterator statesForGtridIt = statesForGtrid.values().iterator();
+        Map<Uid, XAResourceHolderState> statesForGtrid = resourceHolder.getXAResourceHolderStatesForGtrid(resourceManager.getGtrid());
 
-        boolean result = false;
-        List exceptions = new ArrayList();
-        List resourceStates = new ArrayList();
-        while (statesForGtridIt.hasNext()) {
-            XAResourceHolderState resourceHolderState = (XAResourceHolderState) statesForGtridIt.next();
+        boolean result = true;
+        List<Exception> exceptions = new ArrayList<Exception>();
+        List<XAResourceHolderState> resourceStates = new ArrayList<XAResourceHolderState>();
+        for (XAResourceHolderState resourceHolderState : statesForGtrid.values()) {
             try {
                 result &= delistResource(resourceHolderState, flag);
             } catch (BitronixSystemException ex) {
@@ -152,15 +190,18 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
             if (status != Status.STATUS_MARKED_ROLLBACK)
                 setStatus(Status.STATUS_MARKED_ROLLBACK);
 
+            String extraErrorDetails = TransactionManagerServices.getExceptionAnalyzer().extractExtraXAExceptionDetails(ex);
             if (BitronixXAException.isUnilateralRollback(ex)) {
                 // The resource unilaterally rolled back here. We have to throw an exception to indicate this but
                 // The signature of this method is inherited from javax.transaction.Transaction. Thereof, we have choice
                 // between creating a sub-exception of SystemException or using a RuntimeException. Is that the best way
                 // forward as this 'hidden' exception can be left throw out at unexpected locations where SystemException
                 // should be rethrown but the exception thrown here should be catched & handled... ?
-                throw new BitronixRollbackSystemException("resource " + resourceHolderState + " unilaterally rolled back, error=" + Decoder.decodeXAExceptionErrorCode(ex), ex);
+                throw new BitronixRollbackSystemException("resource " + resourceHolderState + " unilaterally rolled back, error=" +
+                        Decoder.decodeXAExceptionErrorCode(ex) + (extraErrorDetails == null ? "" : ", extra error=" + extraErrorDetails), ex);
             }
-            throw new BitronixSystemException("cannot delist " + resourceHolderState + ", error=" + Decoder.decodeXAExceptionErrorCode(ex), ex);
+            throw new BitronixSystemException("cannot delist " + resourceHolderState + ", error=" + Decoder.decodeXAExceptionErrorCode(ex) +
+                    (extraErrorDetails == null ? "" : ", extra error=" + extraErrorDetails), ex);
         }
     }
 
@@ -176,7 +217,7 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
         synchronizationScheduler.add(synchronization, Scheduler.DEFAULT_POSITION);
     }
 
-    public Scheduler getSynchronizationScheduler() {
+    public Scheduler<Synchronization> getSynchronizationScheduler() {
         return synchronizationScheduler;
     }
 
@@ -186,11 +227,19 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
         if (isDone())
             throw new IllegalStateException("transaction is done, cannot commit it");
 
-        TransactionManagerServices.getTaskScheduler().cancelTransactionTimeout(this);
+        taskScheduler.cancelTransactionTimeout(this);
 
         // beforeCompletion must be called before the check to STATUS_MARKED_ROLLBACK as the synchronization
         // can still set the status to STATUS_MARKED_ROLLBACK.
-        fireBeforeCompletionEvent();
+        try {
+            fireBeforeCompletionEvent();
+        } catch (BitronixSystemException ex) {
+            rollback();
+            throw new BitronixRollbackException("SystemException thrown during beforeCompletion cycle caused transaction rollback", ex);
+        } catch (RuntimeException ex) {
+            rollback();
+            throw new BitronixRollbackException("RuntimeException thrown during beforeCompletion cycle caused transaction rollback", ex);
+        }
 
         // The following if statements and try/catch block must not be included in the prepare try-catch block as
         // they call rollback().
@@ -206,6 +255,7 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
         } catch (BitronixRollbackException ex) {
             if (log.isDebugEnabled()) log.debug("delistment error causing transaction rollback", ex);
             rollback();
+            // the caught BitronixRollbackException's message is pre-formatted to be appended to this message
             throw new BitronixRollbackException("delistment error caused transaction rollback" + ex.getMessage());
         }
 
@@ -216,7 +266,7 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
         }
 
         try {
-            List interestedResources;
+            List<XAResourceHolderState> interestedResources;
 
             // prepare phase
             try {
@@ -238,6 +288,10 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
 
             committer.commit(this, interestedResources);
 
+            if (resourceManager.size() == 0 && TransactionManagerServices.getConfiguration().isDebugZeroResourceTransaction()) {
+                log.warn(buildZeroTransactionDebugMessage(activationStackTrace, new StackTrace()));
+            }
+
             if (log.isDebugEnabled()) log.debug("successfully committed " + this);
         }
         finally {
@@ -251,7 +305,7 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
         if (isDone())
             throw new IllegalStateException("transaction is done, cannot roll it back");
 
-        TransactionManagerServices.getTaskScheduler().cancelTransactionTimeout(this);
+        taskScheduler.cancelTransactionTimeout(this);
 
         try {
             delistUnclosedResources(XAResource.TMSUCCESS);
@@ -263,12 +317,11 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
             try {
                 if (log.isDebugEnabled()) log.debug("rolling back, " + resourceManager.size() + " enlisted resource(s)");
 
-                List resourcesToRollback = new ArrayList();
-                List allResources = resourceManager.getAllResources();
-                for (int i = 0; i < allResources.size(); i++) {
-                    XAResourceHolderState resourceHolderState = (XAResourceHolderState) allResources.get(i);
-                    if (!resourceHolderState.isFailed())
-                        resourcesToRollback.add(resourceHolderState);
+                List<XAResourceHolderState> resourcesToRollback = new ArrayList<XAResourceHolderState>();
+                List<XAResourceHolderState> allResources = resourceManager.getAllResources();
+                for (XAResourceHolderState resource : allResources) {
+                    if (!resource.isFailed())
+                        resourcesToRollback.add(resource);
                 }
 
                 rollbacker.rollback(this, resourcesToRollback);
@@ -312,10 +365,13 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
             throw new IllegalStateException("transaction has already started");
 
         setStatus(Status.STATUS_ACTIVE);
-        this.startDate = new Date();
-        this.timeoutDate = new Date(System.currentTimeMillis() + (timeout * 1000L));
+        this.startDate = new Date(MonotonicClock.currentTimeMillis());
+        this.timeoutDate = new Date(MonotonicClock.currentTimeMillis() + (timeout * 1000L));
+        if (TransactionManagerServices.getConfiguration().isDebugZeroResourceTransaction()) {
+            this.activationStackTrace = new StackTrace();
+        }
 
-        TransactionManagerServices.getTaskScheduler().scheduleTransactionTimeout(this, timeoutDate);
+        taskScheduler.scheduleTransactionTimeout(this, timeoutDate);
     }
 
 
@@ -323,14 +379,13 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
         setStatus(status, resourceManager.collectUniqueNames());
     }
 
-    public void setStatus(int status, Set uniqueNames) throws BitronixSystemException {
+    public void setStatus(int status, Set<String> uniqueNames) throws BitronixSystemException {
         try {
             boolean force = (resourceManager.size() > 1) && (status == Status.STATUS_COMMITTING);
             if (log.isDebugEnabled()) log.debug("changing transaction status to " + Decoder.decodeStatus(status) + (force ? " (forced)" : ""));
 
             int oldStatus = this.status;
             this.status = status;
-            Journal journal = TransactionManagerServices.getJournal();
             journal.log(status, resourceManager.getGtrid(), uniqueNames);
             if (force) {
                 journal.force();
@@ -349,9 +404,8 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
     private void fireTransactionStatusChangedEvent(int oldStatus, int newStatus) {
         if (log.isDebugEnabled()) log.debug("transaction status is changing from " + Decoder.decodeStatus(oldStatus) + " to " +
                 Decoder.decodeStatus(newStatus) + " - executing " + transactionStatusListeners.size() + " listener(s)");
-        
-        for (int i = 0; i < transactionStatusListeners.size(); i++) {
-            TransactionStatusChangeListener listener = (TransactionStatusChangeListener) transactionStatusListeners.get(i);
+
+        for (TransactionStatusChangeListener listener : transactionStatusListeners) {
             if (log.isDebugEnabled()) log.debug("executing TransactionStatusChangeListener " + listener);
             listener.statusChanged(oldStatus, newStatus);
             if (log.isDebugEnabled()) log.debug("executed TransactionStatusChangeListener " + listener);
@@ -391,39 +445,39 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
      * @throws bitronix.tm.internal.BitronixRollbackException if some resources unilaterally rolled back before end() call.
      */
     private void delistUnclosedResources(int flag) throws BitronixRollbackException {
-        List resources = resourceManager.getAllResources();
-        List rolledBackResources = new ArrayList();
-        List failedResources = new ArrayList();
+        List<XAResourceHolderState> allResources = resourceManager.getAllResources();
+        List<XAResourceHolderState> rolledBackResources = new ArrayList<XAResourceHolderState>();
+        List<XAResourceHolderState> failedResources = new ArrayList<XAResourceHolderState>();
 
-        for (int i = 0; i < resources.size(); i++) {
-            XAResourceHolderState resourceHolderState = (XAResourceHolderState) resources.get(i);
-            if (!resourceHolderState.isEnded()) {
-                if (log.isDebugEnabled()) log.debug("found unclosed resource to delist: " + resourceHolderState);
+        for (XAResourceHolderState resource : allResources) {
+            if (!resource.isEnded()) {
+                if (log.isDebugEnabled()) log.debug("found unclosed resource to delist: " + resource);
                 try {
-                    delistResource(resourceHolderState, flag);
+                    delistResource(resource, flag);
                 } catch (BitronixRollbackSystemException ex) {
-                    rolledBackResources.add(resourceHolderState);
-                    if (log.isDebugEnabled()) log.debug("resource unilaterally rolled back: " + resourceHolderState, ex);
+                    rolledBackResources.add(resource);
+                    if (log.isDebugEnabled())
+                        log.debug("resource unilaterally rolled back: " + resource, ex);
                 } catch (SystemException ex) {
-                    failedResources.add(resourceHolderState);
-                    log.warn("error delisting resource, assuming unilateral rollback: " + resourceHolderState, ex);
+                    failedResources.add(resource);
+                    log.warn("error delisting resource, assuming unilateral rollback: " + resource, ex);
                 }
-            }
-            else
-                if (log.isDebugEnabled()) log.debug("no need to delist already closed resource: " + resourceHolderState);
+            } else if (log.isDebugEnabled())
+                log.debug("no need to delist already closed resource: " + resource);
         } // for
 
         if (!rolledBackResources.isEmpty() || !failedResources.isEmpty()) {
-            StringBuffer sb = new StringBuffer();
+            String lineSeparator = System.getProperty("line.separator");
+            StringBuilder sb = new StringBuilder();
             if (!rolledBackResources.isEmpty()) {
-                sb.append(System.getProperty("line.separator"));
+                sb.append(lineSeparator);
                 sb.append("  resource(s) ");
                 sb.append(Decoder.collectResourcesNames(rolledBackResources));
                 sb.append(" unilaterally rolled back");
 
             }
             if (!failedResources.isEmpty()) {
-                sb.append(System.getProperty("line.separator"));
+                sb.append(lineSeparator);
                 sb.append("  resource(s) ");
                 sb.append(Decoder.collectResourcesNames(failedResources));
                 sb.append(" could not be delisted");
@@ -441,7 +495,7 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
      * @throws BitronixSystemException when a resource could not rollback prepapared state.
      */
     private void rollbackPrepareFailure(RollbackException rbEx) throws BitronixSystemException {
-        List interestedResources = resourceManager.getAllResources();
+        List<XAResourceHolderState> interestedResources = resourceManager.getAllResources();
         try {
             rollbacker.rollback(this, interestedResources);
             if (log.isDebugEnabled()) log.debug("rollback after prepare failure succeeded");
@@ -450,8 +504,8 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
             PhaseException preparePhaseEx = (PhaseException) rbEx.getCause();
             PhaseException rollbackPhaseEx = (PhaseException) ex.getCause();
 
-            List exceptions = new ArrayList();
-            List resources = new ArrayList();
+            List<Exception> exceptions = new ArrayList<Exception>();
+            List<XAResourceHolderState> resources = new ArrayList<XAResourceHolderState>();
 
             exceptions.addAll(preparePhaseEx.getExceptions());
             exceptions.addAll(rollbackPhaseEx.getExceptions());
@@ -489,9 +543,7 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
         getResourceManager().clearXAResourceHolderStates();
 
         if (log.isDebugEnabled()) log.debug("after completion, " + synchronizationScheduler.size() + " synchronization(s) to execute");
-        Iterator it = synchronizationScheduler.iterator();
-        while (it.hasNext()) {
-            Synchronization synchronization = (Synchronization) it.next();
+        for (Synchronization synchronization : synchronizationScheduler) {
             try {
                 if (log.isDebugEnabled()) log.debug("executing synchronization " + synchronization + " with status=" + Decoder.decodeStatus(status));
                 synchronization.afterCompletion(status);
@@ -503,6 +555,17 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
         ManagementRegistrar.unregister("bitronix.tm:type=Transaction,Gtrid=" + resourceManager.getGtrid());
     }
 
+    static String buildZeroTransactionDebugMessage(StackTrace activationStackTrace, StackTrace commitStackTrace) {
+        String lineSeparator = System.getProperty("line.separator");
+        final StringBuilder sb = new StringBuilder();
+        sb.append("committed transaction with 0 enlisted resource").append(lineSeparator);
+        sb.append("==================== Began at ====================").append(lineSeparator);
+        sb.append(ExceptionUtils.getStackTrace(activationStackTrace)).append(lineSeparator);
+        sb.append("==================== Committed at ====================").append(lineSeparator);
+        sb.append(ExceptionUtils.getStackTrace(commitStackTrace)).append(lineSeparator);
+        return sb.toString();
+    }
+
     private boolean isDone() {
         switch (status) {
             case Status.STATUS_PREPARING:
@@ -548,4 +611,13 @@ public class BitronixTransaction implements Transaction, BitronixTransactionMBea
     public Date getStartDate() {
         return startDate;
     }
+
+    /**
+     * Returns the activation {@link StackTrace} if it is available.
+     *
+     * @return the call stack of where the transaction began
+     */
+    StackTrace getActivationStackTrace() {
+        return activationStackTrace;
+    }
 }
