@@ -49,27 +49,29 @@ public class Configuration implements Service {
 
     private final static int MAX_SERVER_ID_LENGTH = 51;
 
-    private String serverId;
-    private byte[] serverIdArray;
-    private String logPart1Filename;
-    private String logPart2Filename;
-    private boolean forcedWriteEnabled;
-    private boolean forceBatchingEnabled;
-    private int maxLogSizeInMb;
-    private boolean filterLogStatus;
-    private boolean skipCorruptedLogs;
-    private boolean asynchronous2Pc;
-    private boolean warnAboutZeroResourceTransaction;
-    private int defaultTransactionTimeout;
-    private int gracefulShutdownInterval;
-    private int backgroundRecoveryIntervalSeconds;
-    private boolean disableJmx;
-    private String jndiUserTransactionName;
-    private String jndiTransactionSynchronizationRegistryName;
-    private String journal;
-    private boolean currentNodeOnlyRecovery;
-    private boolean allowMultipleLrc;
-    private String resourceConfigurationFilename;
+    private volatile String serverId;
+    private volatile byte[] serverIdArray;
+    private volatile String logPart1Filename;
+    private volatile String logPart2Filename;
+    private volatile boolean forcedWriteEnabled;
+    private volatile boolean forceBatchingEnabled;
+    private volatile int maxLogSizeInMb;
+    private volatile boolean filterLogStatus;
+    private volatile boolean skipCorruptedLogs;
+    private volatile boolean asynchronous2Pc;
+    private volatile boolean warnAboutZeroResourceTransaction;
+    private volatile boolean debugZeroResourceTransaction;
+    private volatile int defaultTransactionTimeout;
+    private volatile int gracefulShutdownInterval;
+    private volatile int backgroundRecoveryIntervalSeconds;
+    private volatile boolean disableJmx;
+    private volatile String jndiUserTransactionName;
+    private volatile String jndiTransactionSynchronizationRegistryName;
+    private volatile String journal;
+    private volatile String exceptionAnalyzer;
+    private volatile boolean currentNodeOnlyRecovery;
+    private volatile boolean allowMultipleLrc;
+    private volatile String resourceConfigurationFilename;
 
 
     protected Configuration() {
@@ -104,6 +106,7 @@ public class Configuration implements Service {
             skipCorruptedLogs = getBoolean(properties, "bitronix.tm.journal.disk.skipCorruptedLogs", false);
             asynchronous2Pc = getBoolean(properties, "bitronix.tm.2pc.async", false);
             warnAboutZeroResourceTransaction = getBoolean(properties, "bitronix.tm.2pc.warnAboutZeroResourceTransactions", true);
+            debugZeroResourceTransaction = getBoolean(properties, "bitronix.tm.2pc.debugZeroResourceTransactions", false);
             defaultTransactionTimeout = getInt(properties, "bitronix.tm.timer.defaultTransactionTimeout", 60);
             gracefulShutdownInterval = getInt(properties, "bitronix.tm.timer.gracefulShutdownInterval", 60);
             backgroundRecoveryIntervalSeconds = getInt(properties, "bitronix.tm.timer.backgroundRecoveryIntervalSeconds", 60);
@@ -111,6 +114,7 @@ public class Configuration implements Service {
             jndiUserTransactionName = getString(properties, "bitronix.tm.jndi.userTransactionName", "java:comp/UserTransaction");
             jndiTransactionSynchronizationRegistryName = getString(properties, "bitronix.tm.jndi.transactionSynchronizationRegistryName", "java:comp/TransactionSynchronizationRegistry");
             journal = getString(properties, "bitronix.tm.journal", "disk");
+            exceptionAnalyzer = getString(properties, "bitronix.tm.exceptionAnalyzer", null);
             currentNodeOnlyRecovery = getBoolean(properties, "bitronix.tm.currentNodeOnlyRecovery", true);
             allowMultipleLrc = getBoolean(properties, "bitronix.tm.allowMultipleLrc", false);
             resourceConfigurationFilename = getString(properties, "bitronix.tm.resource.configuration", null);
@@ -345,6 +349,32 @@ public class Configuration implements Service {
         return this;
     }
 
+    /**
+     * Should creation and commit call stacks of transactions executed without a single enlisted tracked and logged
+     * or not?
+     * <p>Property name:<br/><b>bitronix.tm.2pc.debugZeroResourceTransactions -</b> <i>(defaults to false)</i></p>
+     * @return true if creation and commit call stacks of transactions executed without a single enlisted resource
+     *         should be tracked and logged.
+     */
+    public boolean isDebugZeroResourceTransaction() {
+        return debugZeroResourceTransaction;
+    }
+
+    /**
+     * Set if creation and commit call stacks of transactions executed without a single enlisted resource should be
+     * tracked and logged.
+     * @see #isDebugZeroResourceTransaction()
+     * @see #isWarnAboutZeroResourceTransaction()
+     * @param debugZeroResourceTransaction true if the creation and commit call stacks of transaction executed without
+     *        a single enlisted resource should be tracked and logged.
+     * @return this.
+     */
+    public Configuration setDebugZeroResourceTransaction(boolean debugZeroResourceTransaction) {
+        checkNotStarted();
+        this.debugZeroResourceTransaction = debugZeroResourceTransaction;
+        return this;
+    }
+
     /**
      * Default transaction timeout in seconds.
      * <p>Property name:<br/><b>bitronix.tm.timer.defaultTransactionTimeout -</b> <i>(defaults to 60)</i></p>
@@ -521,6 +551,26 @@ public class Configuration implements Service {
         return this;
     }
 
+    /**
+     * Get the exception analyzer implementation. Can be <code>null</code> for the default one or a class name.
+     * @return the exception analyzer name.
+     */
+    public String getExceptionAnalyzer() {
+        return exceptionAnalyzer;
+    }
+
+    /**
+     * Set the exception analyzer implementation. Can be <code>null</code> for the default one or a class name.
+     * @see #getExceptionAnalyzer()
+     * @param exceptionAnalyzer the exception analyzer name.
+     * @return this.
+     */
+    public Configuration setExceptionAnalyzer(String exceptionAnalyzer) {
+        checkNotStarted();
+        this.exceptionAnalyzer = exceptionAnalyzer;
+        return this;
+    }
+
     /**
      * Should the recovery process <b>not</b> recover XIDs generated with another JVM unique ID? Setting this property to true
      * is useful in clustered environments where multiple instances of BTM are running on different nodes.
@@ -622,10 +672,11 @@ public class Configuration implements Service {
     }
 
     public void shutdown() {
+        serverIdArray = null;
     }
 
     public String toString() {
-        StringBuffer sb = new StringBuffer(512);
+        StringBuilder sb = new StringBuilder(512);
         sb.append("a Configuration with [");
 
         try {
@@ -659,7 +710,7 @@ public class Configuration implements Service {
     }
 
     static boolean getBoolean(Properties properties, String key, boolean defaultValue) {
-        return Boolean.valueOf(getString(properties, key, "" + defaultValue)).booleanValue();
+        return Boolean.valueOf(getString(properties, key, "" + defaultValue));
     }
 
     static int getInt(Properties properties, String key, int defaultValue) {
