@@ -40,6 +40,7 @@ import javax.transaction.xa.XAResource;
 import javax.transaction.xa.Xid;
 import java.io.IOException;
 import java.util.*;
+import java.util.concurrent.atomic.AtomicBoolean;
 
 /**
  * Recovery process implementation. Here is Mike Spille's description of XA recovery:
@@ -86,8 +87,8 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
 
     private final static Logger log = LoggerFactory.getLogger(Recoverer.class);
 
-    private final Map registeredResources = new HashMap();
-    private final Map recoveredXidSets = new HashMap();
+    private final Map<String, XAResourceProducer> registeredResources = new HashMap<String, XAResourceProducer>();
+    private final Map<String, Set<BitronixXid>> recoveredXidSets = new HashMap<String, Set<BitronixXid>>();
 
     private volatile Exception completionException;
     private volatile int committedCount;
@@ -123,11 +124,12 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
             rolledbackCount = 0;
             long oldestTransactionTimestamp = Long.MAX_VALUE;
 
+            // Collect dangling records from journal, must run before oldestTransactionTimestamp is calculated
+            Map<Uid, TransactionLogRecord> danglingRecords = TransactionManagerServices.getJournal().collectDanglingRecords();
+
             // Query resources from ResourceRegistrar
             synchronized (ResourceRegistrar.class) {
-                Iterator it = ResourceRegistrar.getResourcesUniqueNames().iterator();
-                while (it.hasNext()) {
-                    String name = (String) it.next();
+                for (String name : ResourceRegistrar.getResourcesUniqueNames()) {
                     registeredResources.put(name, ResourceRegistrar.get(name));
                 }
 
@@ -136,13 +138,11 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
                 }
             }
 
-            Map danglingRecords = TransactionManagerServices.getJournal().collectDanglingRecords();
-
             // 1. call recover on all known resources
             recoverAllResources();
 
             // 2. commit dangling COMMITTING transactions
-            Set committedGtrids = commitDanglingTransactions(oldestTransactionTimestamp, danglingRecords);
+            Set<Uid> committedGtrids = commitDanglingTransactions(oldestTransactionTimestamp, danglingRecords);
             committedCount = committedGtrids.size();
 
             // 3. rollback any remaining recovered transaction
@@ -216,22 +216,23 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
      * Step 1.
      */
     private void recoverAllResources() {
-        Iterator it = new HashMap(registeredResources).entrySet().iterator(); // a cloned registeredResources Map must be iterated as the original one can be modified in the loop
-        while (it.hasNext()) {
-            Map.Entry entry = (Map.Entry) it.next();
-            String uniqueName = (String) entry.getKey();
-            XAResourceProducer producer = (XAResourceProducer) entry.getValue();
+        // a cloned registeredResources Map must be iterated as the original one can be modified in the loop
+        for (Map.Entry<String, XAResourceProducer> entry : new HashMap<String, XAResourceProducer>(registeredResources).entrySet()) {
+            String uniqueName = entry.getKey();
+            XAResourceProducer producer = entry.getValue();
 
             try {
                 if (log.isDebugEnabled()) log.debug("performing recovery on " + uniqueName);
-                Set xids = recover(producer);
+                Set<BitronixXid> xids = recover(producer);
                 if (log.isDebugEnabled()) log.debug("recovered " + xids.size() + " XID(s) from resource " + uniqueName);
                 recoveredXidSets.put(uniqueName, xids);
                 producer.setFailed(false);
             } catch (XAException ex) {
                 producer.setFailed(true);
                 registeredResources.remove(uniqueName);
-                log.warn("error running recovery on resource '" + uniqueName + "', resource marked as failed (background recoverer will retry recovery) (error=" + Decoder.decodeXAExceptionErrorCode(ex) + ")", ex);
+                String extraErrorDetails = TransactionManagerServices.getExceptionAnalyzer().extractExtraXAExceptionDetails(ex);
+                log.warn("error running recovery on resource '" + uniqueName + "', resource marked as failed (background recoverer will retry recovery)" +
+                        " (error=" + Decoder.decodeXAExceptionErrorCode(ex) + ")" + (extraErrorDetails == null ? "" : ", extra error=" + extraErrorDetails), ex);
             } catch (Exception ex) {
                 producer.setFailed(true);
                 registeredResources.remove(uniqueName);
@@ -248,7 +249,7 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
      * @throws javax.transaction.xa.XAException if {@link XAResource#recover(int)} call fails.
      * @throws RecoveryException if an error preventing recovery happened.
      */
-    private Set recover(XAResourceProducer producer) throws XAException, RecoveryException {
+    private Set<BitronixXid> recover(XAResourceProducer producer) throws XAException, RecoveryException {
         if (producer == null)
             throw new IllegalArgumentException("recoverable resource cannot be null");
 
@@ -263,6 +264,7 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
 
     /**
      * Commit transactions that have a dangling COMMITTING record in the journal.
+     * Transactions younger than oldestTransactionTimestamp are ignored.
      * Step 2.
      * @param oldestTransactionTimestamp the timestamp of the oldest transaction still in-flight.
      * @param danglingRecords a Map using Uid objects GTRID as key and {@link TransactionLogRecord} as value.
@@ -270,18 +272,16 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
      * @throws java.io.IOException if there is an I/O error reading the journal.
      * @throws RecoveryException if an error preventing recovery happened.
      */
-    private Set commitDanglingTransactions(long oldestTransactionTimestamp, Map danglingRecords) throws IOException, RecoveryException {
-        Set committedGtrids = new HashSet();
+    private Set<Uid> commitDanglingTransactions(long oldestTransactionTimestamp, Map<Uid, TransactionLogRecord> danglingRecords) throws IOException, RecoveryException {
+        Set<Uid> committedGtrids = new HashSet<Uid>();
 
         if (log.isDebugEnabled()) log.debug("found " + danglingRecords.size() + " dangling record(s) in journal");
-        Iterator it = danglingRecords.entrySet().iterator();
-        while (it.hasNext()) {
-            Map.Entry entry = (Map.Entry) it.next();
-            Uid gtrid = (Uid) entry.getKey();
-            TransactionLogRecord tlog = (TransactionLogRecord) entry.getValue();
+        for (Map.Entry<Uid, TransactionLogRecord> entry : danglingRecords.entrySet()) {
+            Uid gtrid = entry.getKey();
+            TransactionLogRecord tlog = entry.getValue();
 
-            Set uniqueNames = tlog.getUniqueNames();
-            Set danglingTransactions = getDanglingTransactionsInRecoveredXids(uniqueNames, tlog.getGtrid());
+            Set<String> uniqueNames = tlog.getUniqueNames();
+            Set<DanglingTransaction> danglingTransactions = getDanglingTransactionsInRecoveredXids(uniqueNames, tlog.getGtrid());
 
             long txTimestamp = gtrid.extractTimestamp();
             if (log.isDebugEnabled()) log.debug("recovered XID timestamp: " + txTimestamp + " - oldest in-flight TX timestamp: " + oldestTransactionTimestamp);
@@ -292,18 +292,16 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
                 if (log.isDebugEnabled()) log.debug("committed dangling transaction with GTRID " + gtrid);
                 committedGtrids.add(gtrid);
 
-                Set participatingUniqueNames = filterParticipatingUniqueNamesInRecoveredXids(uniqueNames);
+                Set<String> participatingUniqueNames = filterParticipatingUniqueNamesInRecoveredXids(uniqueNames);
 
                 if (participatingUniqueNames.size() > 0) {
                     if (log.isDebugEnabled()) log.debug("updating journal's transaction with GTRID " + gtrid + " status to COMMITTED for names [" + buildUniqueNamesString(participatingUniqueNames) + "]");
                     TransactionManagerServices.getJournal().log(Status.STATUS_COMMITTED, tlog.getGtrid(), participatingUniqueNames);
-                }
-                else {
+                } else {
                     if (log.isDebugEnabled()) log.debug("not updating journal's transaction with GTRID " + gtrid + " status to COMMITTED as no resource could be found (incremental recovery will need to clean this)");
                     committedGtrids.remove(gtrid);
                 }
-            }
-            else {
+            } else {
                 if (log.isDebugEnabled()) log.debug("skipping in-flight transaction with GTRID " + gtrid);
             }
         }
@@ -320,44 +318,37 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
      * @param gtrid the GTRID to look for.
      * @return a set of {@link DanglingTransaction}s.
      */
-    private Set getDanglingTransactionsInRecoveredXids(Set uniqueNames, Uid gtrid) {
-        Set danglingTransactions = new HashSet();
+    private Set<DanglingTransaction> getDanglingTransactionsInRecoveredXids(Set<String> uniqueNames, Uid gtrid) {
+        Set<DanglingTransaction> danglingTransactions = new HashSet<DanglingTransaction>();
 
-        Iterator it = uniqueNames.iterator();
-        while (it.hasNext()) {
-            String uniqueName = (String) it.next();
+        for (String uniqueName : uniqueNames) {
             if (log.isDebugEnabled()) log.debug("finding dangling transaction(s) in recovered XID(s) of resource " + uniqueName);
-            Set recoveredXids = (Set) recoveredXidSets.get(uniqueName);
+            Set<BitronixXid> recoveredXids = recoveredXidSets.get(uniqueName);
             if (recoveredXids == null) {
                 if (log.isDebugEnabled()) log.debug("resource " + uniqueName + " did not recover, skipping commit");
                 continue;
             }
 
-            Iterator it2 = recoveredXids.iterator();
-            while (it2.hasNext()) {
-                BitronixXid recoveredXid = (BitronixXid) it2.next();
+            for (BitronixXid recoveredXid : recoveredXids) {
                 if (gtrid.equals(recoveredXid.getGlobalTransactionIdUid())) {
                     if (log.isDebugEnabled()) log.debug("found a recovered XID matching dangling log's GTRID " + gtrid + " in resource " + uniqueName);
                     danglingTransactions.add(new DanglingTransaction(uniqueName, recoveredXid));
                 }
-            } // while it2.hasNext()
+            }
         }
 
         return danglingTransactions;
     }
 
-    private Set filterParticipatingUniqueNamesInRecoveredXids(Set uniqueNames) {
-        Set recoveredUniqueNames = new HashSet();
+    private Set<String> filterParticipatingUniqueNamesInRecoveredXids(Set<String> uniqueNames) {
+        Set<String> recoveredUniqueNames = new HashSet<String>();
 
-        Iterator it = uniqueNames.iterator();
-        while (it.hasNext()) {
-            String uniqueName = (String) it.next();
+        for (String uniqueName : uniqueNames) {
             if (log.isDebugEnabled()) log.debug("finding dangling transaction(s) in recovered XID(s) of resource " + uniqueName);
-            Set recoveredXids = (Set) recoveredXidSets.get(uniqueName);
+            Set<BitronixXid> recoveredXids = recoveredXidSets.get(uniqueName);
             if (recoveredXids == null) {
                 if (log.isDebugEnabled()) log.debug("cannot find resource '" + uniqueName + "' present in the journal, leaving it for incremental recovery");
-            }
-            else {
+            } else {
                 recoveredUniqueNames.add(uniqueName);
             }
         }
@@ -371,12 +362,10 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
      * @param danglingTransactions a set of {@link DanglingTransaction}s to commit.
      * @throws RecoveryException if an error preventing recovery happened.
      */
-    private void commit(Set danglingTransactions) throws RecoveryException {
+    private void commit(Set<DanglingTransaction> danglingTransactions) throws RecoveryException {
         if (log.isDebugEnabled()) log.debug(danglingTransactions.size() + " branch(es) to commit");
 
-        Iterator it = danglingTransactions.iterator();
-        while (it.hasNext()) {
-            DanglingTransaction danglingTransaction = (DanglingTransaction) it.next();
+        for (DanglingTransaction danglingTransaction : danglingTransactions) {
             Xid xid = danglingTransaction.getXid();
             String uniqueName = danglingTransaction.getUniqueName();
 
@@ -394,7 +383,7 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
      * @throws RecoveryException if an error preventing recovery happened.
      */
     private boolean commit(String uniqueName, Xid xid) throws RecoveryException {
-        XAResourceProducer producer = (XAResourceProducer) registeredResources.get(uniqueName);
+        XAResourceProducer producer = registeredResources.get(uniqueName);
         try {
             XAResourceHolderState xaResourceHolderState = producer.startRecovery();
             return RecoveryHelper.commit(xaResourceHolderState, xid);
@@ -412,14 +401,12 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
      * @return the rolled back branches count.
      * @throws RecoveryException if an error preventing recovery happened.
      */
-    private int rollbackAbortedTransactions(long oldestTransactionTimestamp, Set committedGtrids) throws RecoveryException {
+    private int rollbackAbortedTransactions(long oldestTransactionTimestamp, Set<Uid> committedGtrids) throws RecoveryException {
         if (log.isDebugEnabled()) log.debug("rolling back aborted branch(es)");
         int rollbackCount = 0;
-        Iterator it = recoveredXidSets.entrySet().iterator();
-        while (it.hasNext()) {
-            Map.Entry entry = (Map.Entry) it.next();
-            String uniqueName = (String) entry.getKey();
-            Set recoveredXids = (Set) entry.getValue();
+        for (Map.Entry<String, Set<BitronixXid>> entry : recoveredXidSets.entrySet()) {
+            String uniqueName = entry.getKey();
+            Set<BitronixXid> recoveredXids = entry.getValue();
 
             if (log.isDebugEnabled()) log.debug("checking " + recoveredXids.size() + " branch(es) on " + uniqueName + " for rollback");
             int count = rollbackAbortedBranchesOfResource(oldestTransactionTimestamp, uniqueName, recoveredXids, committedGtrids);
@@ -441,11 +428,9 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
      * @return the rolled back branches count.
      * @throws RecoveryException if an error preventing recovery happened.
      */
-    private int rollbackAbortedBranchesOfResource(long oldestTransactionTimestamp, String uniqueName, Set recoveredXids, Set committedGtrids) throws RecoveryException {
+    private int rollbackAbortedBranchesOfResource(long oldestTransactionTimestamp, String uniqueName, Set<BitronixXid> recoveredXids, Set<Uid> committedGtrids) throws RecoveryException {
         int abortedCount = 0;
-        Iterator it = recoveredXids.iterator();
-        while (it.hasNext()) {
-            BitronixXid recoveredXid = (BitronixXid) it.next();
+        for (BitronixXid recoveredXid : recoveredXids) {
             if (committedGtrids.contains(recoveredXid.getGlobalTransactionIdUid())) {
                 if (log.isDebugEnabled()) log.debug("XID has been committed, skipping rollback: " + recoveredXid + " on " + uniqueName);
                 continue;
@@ -475,7 +460,7 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
      * @throws RecoveryException if an error preventing recovery happened.
      */
     private boolean rollback(String uniqueName, Xid xid) throws RecoveryException {
-        XAResourceProducer producer = (XAResourceProducer) registeredResources.get(uniqueName);
+        XAResourceProducer producer = registeredResources.get(uniqueName);
         if (producer == null) {
             if (log.isDebugEnabled()) log.debug("resource " + uniqueName + " has not recovered, skipping rollback");
             return false;
@@ -497,11 +482,11 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
         return buildUniqueNamesString(registeredResources.keySet());
     }
 
-    private static String buildUniqueNamesString(Set uniqueNames) {
-        StringBuffer resourcesUniqueNames = new StringBuffer();
-        Iterator it = uniqueNames.iterator();
+    private static String buildUniqueNamesString(Set<String> uniqueNames) {
+        StringBuilder resourcesUniqueNames = new StringBuilder();
+        Iterator<String> it = uniqueNames.iterator();
         while (it.hasNext()) {
-            String uniqueName = (String) it.next();
+            String uniqueName = it.next();
             resourcesUniqueNames.append(uniqueName);
             if (it.hasNext())
                 resourcesUniqueNames.append(", ");
@@ -509,40 +494,4 @@ public class Recoverer implements Runnable, Service, RecovererMBean {
         return resourcesUniqueNames.toString();
     }
 
-    /**
-     * A boolean value that may be updated atomically. This is a simplified subset of the JDK 1.5+
-     * java.util.concurrent.atomic.AtomicBoolean class.
-     */
-    private static class AtomicBoolean {
-        private boolean value;
-
-        public AtomicBoolean(boolean value) {
-            this.value = value;
-        }
-
-        public synchronized boolean get() {
-            return value;
-        }
-
-        public synchronized void set(boolean value) {
-            this.value = value;
-        }
-
-        /**
-         * Atomically sets the value to the given updated value if the current value == the expected value.
-         *
-         * @param expect the expected value.
-         * @param update the new value.
-         * @return true if successful. False return indicates that the actual value was not equal to the expected value.
-         */
-        public synchronized boolean compareAndSet(boolean expect, boolean update) {
-            if (this.value == expect) {
-                this.value = update;
-                return true;
-            }
-            return false;
-        }
-
-    }
-
 }
