@@ -44,8 +44,9 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
 
     private final static Logger log = LoggerFactory.getLogger(JdbcConnectionHandle.class);
 
-    private JdbcPooledConnection jdbcPooledConnection;
-    private Connection delegate;
+    private final JdbcPooledConnection jdbcPooledConnection;
+    private final Connection delegate;
+    private volatile boolean closed = false;
 
     public JdbcConnectionHandle(JdbcPooledConnection jdbcPooledConnection, Connection connection) {
         this.jdbcPooledConnection = jdbcPooledConnection;
@@ -57,7 +58,7 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
     }
 
     private Connection getDelegate() throws SQLException {
-        if (jdbcPooledConnection == null)
+        if (closed)
             throw new SQLException("connection is closed");
         return delegate;
     }
@@ -76,7 +77,7 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
      * @throws SQLException thrown when an error occurs during elistment.
      */
     private void enlistResource() throws SQLException {
-        if (jdbcPooledConnection == null)
+        if (closed)
             throw new SQLException("connection handle already closed");
 
         if (jdbcPooledConnection.getPoolingDataSource().getAutomaticEnlistingEnabled()) {
@@ -96,15 +97,15 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
         if (log.isDebugEnabled()) log.debug("closing " + this);
 
         // in case the connection has already been closed
-        if (jdbcPooledConnection == null)
+        if (closed)
             return;
 
         jdbcPooledConnection.release();
-        jdbcPooledConnection = null;
+        closed = true;
     }
 
     public void commit() throws SQLException {
-        if (jdbcPooledConnection == null)
+        if (closed)
             throw new SQLException("connection handle already closed");
         if (jdbcPooledConnection.isParticipatingInActiveGlobalTransaction())
             throw new SQLException("cannot commit a resource enlisted in a global transaction");
@@ -113,7 +114,7 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
     }
 
     public void rollback() throws SQLException {
-        if (jdbcPooledConnection == null)
+        if (closed)
             throw new SQLException("connection handle already closed");
         if (jdbcPooledConnection.isParticipatingInActiveGlobalTransaction())
             throw new SQLException("cannot rollback a resource enlisted in a global transaction");
@@ -122,7 +123,7 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
     }
 
     public void rollback(Savepoint savepoint) throws SQLException {
-        if (jdbcPooledConnection == null)
+        if (closed)
             throw new SQLException("connection handle already closed");
         if (jdbcPooledConnection.isParticipatingInActiveGlobalTransaction())
             throw new SQLException("cannot rollback a resource enlisted in a global transaction");
@@ -131,7 +132,7 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
     }
 
     public Savepoint setSavepoint() throws SQLException {
-        if (jdbcPooledConnection == null)
+        if (closed)
             throw new SQLException("connection handle already closed");
         if (jdbcPooledConnection.isParticipatingInActiveGlobalTransaction())
             throw new SQLException("cannot set a savepoint on a resource enlisted in a global transaction");
@@ -140,7 +141,7 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
     }
 
     public Savepoint setSavepoint(String name) throws SQLException {
-        if (jdbcPooledConnection == null)
+        if (closed)
             throw new SQLException("connection handle already closed");
         if (jdbcPooledConnection.isParticipatingInActiveGlobalTransaction())
             throw new SQLException("cannot set a savepoint on a resource enlisted in a global transaction");
@@ -149,7 +150,7 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
     }
 
     public void releaseSavepoint(Savepoint savepoint) throws SQLException {
-        if (jdbcPooledConnection == null)
+        if (closed)
             throw new SQLException("connection handle already closed");
         if (jdbcPooledConnection.isParticipatingInActiveGlobalTransaction())
             throw new SQLException("cannot release a savepoint on a resource enlisted in a global transaction");
@@ -158,7 +159,7 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
     }
 
     public boolean getAutoCommit() throws SQLException {
-        if (jdbcPooledConnection == null)
+        if (closed)
             throw new SQLException("connection handle already closed");
 
         if (jdbcPooledConnection.isParticipatingInActiveGlobalTransaction())
@@ -168,7 +169,7 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
     }
 
     public void setAutoCommit(boolean autoCommit) throws SQLException {
-        if (jdbcPooledConnection == null)
+        if (closed)
             throw new SQLException("connection handle already closed");
 
         if (!jdbcPooledConnection.isParticipatingInActiveGlobalTransaction())
@@ -178,9 +179,7 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
     }
 
     public boolean isClosed() throws SQLException {
-        if (jdbcPooledConnection == null)
-            return true;
-        return getDelegate().isClosed();
+        return closed || getDelegate().isClosed();
     }
 
     public Statement createStatement() throws SQLException {
@@ -379,19 +378,16 @@ public class JdbcConnectionHandle extends BaseProxyHandlerClass { // implements
 
     /* java.sql.Wrapper implementation */
 
-    public boolean isWrapperFor(Class iface) throws SQLException {
-        if (Connection.class.equals(iface)) {
-            return true;
-        }
-        return false;
+    public boolean isWrapperFor(Class<?> iface) throws SQLException {
+        return iface.isAssignableFrom(delegate.getClass());
     }
 
-    public Object unwrap(Class iface) throws SQLException {
-        if (Connection.class.equals(iface)) {
-            return delegate;
-        }
-        throw new SQLException(getClass().getName() + " is not a wrapper for interface " + iface.getName());
-    }
+    public <T> T unwrap(Class<T> iface) throws SQLException {
+        if (iface.isAssignableFrom(delegate.getClass())) {
+            return (T) delegate;
+	    }
+	    throw new SQLException(getClass().getName() + " is not a wrapper for " + iface);
+	}
 
     /* BaseProxyHandler implementation */
 
