@@ -1,32 +1,45 @@
+/*
+ * Bitronix Transaction Manager
+ *
+ * Copyright (c) 2010, Bitronix Software.
+ *
+ * This copyrighted material is made available to anyone wishing to use, modify,
+ * copy, or redistribute it subject to the terms and conditions of the GNU
+ * Lesser General Public License, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
+ * for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this distribution; if not, write to:
+ * Free Software Foundation, Inc.
+ * 51 Franklin Street, Fifth Floor
+ * Boston, MA 02110-1301 USA
+ */
 package bitronix.tm.mock;
 
-import bitronix.tm.BitronixTransactionManager;
-import bitronix.tm.TransactionManagerServices;
-import bitronix.tm.BitronixTransaction;
-import bitronix.tm.mock.events.*;
-import bitronix.tm.mock.resource.jdbc.MockXAConnection;
-import bitronix.tm.mock.resource.jdbc.MockDriver;
-import bitronix.tm.mock.resource.MockXAResource;
-import bitronix.tm.resource.jdbc.JdbcConnectionHandle;
-import bitronix.tm.resource.jdbc.JdbcPooledConnection;
-import bitronix.tm.resource.jdbc.PoolingDataSource;
-import bitronix.tm.resource.jdbc.lrc.LrcXADataSource;
-import bitronix.tm.resource.common.XAPool;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
-
-import javax.transaction.*;
-import javax.transaction.xa.XAException;
-import javax.transaction.xa.XAResource;
-import java.sql.Connection;
-import java.sql.SQLException;
-import java.util.List;
-import java.util.ArrayList;
-import java.lang.reflect.Field;
 import java.io.*;
+import java.lang.reflect.*;
+import java.sql.*;
+import java.util.*;
+
+import javax.sql.XAConnection;
+import javax.transaction.*;
+import javax.transaction.xa.*;
+
+import org.slf4j.*;
+
+import bitronix.tm.*;
+import bitronix.tm.mock.events.*;
+import bitronix.tm.mock.resource.MockXAResource;
+import bitronix.tm.mock.resource.jdbc.MockDriver;
+import bitronix.tm.resource.common.XAPool;
+import bitronix.tm.resource.jdbc.*;
+import bitronix.tm.resource.jdbc.lrc.LrcXADataSource;
 
 /**
- * (c) Bitronix, 20-oct.-2005
  *
  * @author lorban
  */
@@ -67,7 +80,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(17, orderedEvents.size());
         int i=0;
@@ -125,7 +138,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(17, orderedEvents.size());
         int i=0;
@@ -189,7 +202,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(17, orderedEvents.size());
         int i=0;
@@ -264,7 +277,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(12, orderedEvents.size());
         int i=0;
@@ -314,7 +327,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(9, orderedEvents.size());
         int i=0;
@@ -358,7 +371,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(9, orderedEvents.size());
         int i=0;
@@ -411,7 +424,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(21, orderedEvents.size());
         int i=0;
@@ -463,7 +476,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(9, orderedEvents.size());
         int i=0;
@@ -524,7 +537,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(14, orderedEvents.size());
         int i=0;
@@ -575,7 +588,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(23, orderedEvents.size());
         int i=0;
@@ -630,7 +643,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(17, orderedEvents.size());
         int i=0;
@@ -679,7 +692,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(17, orderedEvents.size());
         int i=0;
@@ -728,7 +741,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(17, orderedEvents.size());
         int i=0;
@@ -756,8 +769,9 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
         tm.begin();
 
         Connection connection1 = poolingDataSource1.getConnection();
-        JdbcPooledConnection pc1 = ((JdbcConnectionHandle) connection1).getPooledConnection();
-            MockXAConnection mockXAConnection1 = (MockXAConnection) getWrappedXAConnectionOf(pc1);
+        JdbcConnectionHandle handle = (JdbcConnectionHandle) Proxy.getInvocationHandler(connection1);
+        JdbcPooledConnection pc1 = handle.getPooledConnection();
+            XAConnection mockXAConnection1 = (XAConnection) getWrappedXAConnectionOf(pc1);
             MockXAResource mockXAResource = (MockXAResource) mockXAConnection1.getXAResource();
             mockXAResource.setCommitException(new XAException(XAException.XA_HEURCOM));
         connection1.createStatement();
@@ -772,7 +786,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(18, orderedEvents.size());
         int i=0;
@@ -814,8 +828,9 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
         tm.begin();
 
         Connection connection1 = poolingDataSource1.getConnection();
-        JdbcPooledConnection pc1 = ((JdbcConnectionHandle) connection1).getPooledConnection();
-            MockXAConnection mockXAConnection1 = (MockXAConnection) getWrappedXAConnectionOf(pc1);
+        JdbcConnectionHandle handle = (JdbcConnectionHandle) Proxy.getInvocationHandler(connection1);
+        JdbcPooledConnection pc1 = handle.getPooledConnection();
+            XAConnection mockXAConnection1 = (XAConnection) getWrappedXAConnectionOf(pc1);
             MockXAResource mockXAResource = (MockXAResource) mockXAConnection1.getXAResource();
             mockXAResource.setRollbackException(new XAException(XAException.XA_HEURRB));
         connection1.createStatement();
@@ -831,7 +846,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
         // check flow
         List orderedEvents = EventRecorder.getOrderedEvents();
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         assertEquals(14, orderedEvents.size());
         int i=0;
@@ -877,7 +892,7 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
             assertEquals(0, TransactionManagerServices.getTransactionManager().getInFlightTransactions().size());
         }
 
-        System.out.println(EventRecorder.dumpToString());
+        log.info(EventRecorder.dumpToString());
 
         List events = EventRecorder.getOrderedEvents();
 
@@ -984,7 +999,6 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
         assertTrue(thread.isSuccesful());
     }
 
-
     static class LooseTransactionThread extends Thread {
 
         static int successes = 0;
@@ -1048,6 +1062,56 @@ public class NewJdbcProperUsageMockTest extends AbstractMockJdbcTest {
 
     }
 
+    public void testNonEnlistingMethodInTxContext() throws Exception {
+        BitronixTransactionManager tm = TransactionManagerServices.getTransactionManager();
+
+        tm.begin();
+        
+        Connection c = poolingDataSource1.getConnection();
+        assertTrue(c.getAutoCommit());
+        c.close();
+
+        tm.commit();
+
+        tm.shutdown();
+    }
+
+    public void testAutoCommitFalseWhenEnlisted() throws Exception {
+        BitronixTransactionManager tm = TransactionManagerServices.getTransactionManager();
+
+        tm.begin();
+
+        Connection c = poolingDataSource1.getConnection();
+        c.prepareStatement("");
+        assertFalse(c.getAutoCommit());
+        c.close();
+
+        tm.commit();
+
+        tm.shutdown();
+    }
+
+    public void testAutoCommitTrueWhenEnlistedButSuspended() throws Exception {
+        BitronixTransactionManager tm = TransactionManagerServices.getTransactionManager();
+
+        tm.begin();
+
+        Connection c = poolingDataSource1.getConnection();
+        c.prepareStatement("");
+
+        Transaction tx = tm.suspend();
+        assertNull(tm.getTransaction());
+
+        assertTrue(c.getAutoCommit());
+
+        tm.resume(tx);
+        c.close();
+
+        tm.commit();
+
+        tm.shutdown();
+    }
+
     public void testSerialization() throws Exception {
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         ObjectOutputStream oos = new ObjectOutputStream(baos);
